import anime from "animejs";
import Anime from "./anime";
import React from "react";

enum StatusSeverity {
    OK = "OK",
    WARNING = "WARN",
    ERROR = "ERR",
}

interface Status {
    severity: StatusSeverity;
    message: string;
}

interface Diagnostic {
    name: string;
    status: Status;
    get_status: () => Status;
}

const default_diagnostics: Diagnostic[] = [
    {
        name: "Example Diagnostic",
        status: {
            severity: StatusSeverity.OK,
            message: "",
        },
        get_status: () => {
            return {
                severity: StatusSeverity.OK,
                message: "This is an example diagnostic message",
            }
        }
    }
];

export default function Diagnostics() {
    const [diagnosticResults, setDiagnosticResults]: [Diagnostic[], React.Dispatch<React.SetStateAction<Diagnostic[]>>]
        = React.useState(default_diagnostics);
    const [diagnostics]: [Diagnostic[], React.Dispatch<React.SetStateAction<Diagnostic[]>>]
        = React.useState(default_diagnostics);

    React.useEffect(() => {
        // Ensure diagnostics is always an array before trying to map
        if (Array.isArray(diagnostics)) {
            console.log("Diagnostics: ", diagnostics);

            const new_diagnostics = diagnostics.map((d) => {
                return {
                    name: d.name,
                    status: d.get_status(),
                    get_status: d.get_status,
                }
            });

            // Update the diagnostics state with the new values
            setDiagnosticResults(new_diagnostics);
        }
    }, [diagnostics]); // This will now only rerun when the state actually changes

    return (
        <div>
            <Anime params={{
                targets: ".text-stagger",
                opacity: [0, 1],
                duration: 1000,
                delay: anime.stagger(300),
            }}>
                <ul>
                    {
                        diagnosticResults.map((d) => {
                            return (
                                <div key={d.name} className="text-stagger">
                                    <h2>{d.name}</h2>
                                    {/* SystemD style */}
                                    <p>
                                        [ <span style={{ 
                                            color: d.status.severity === StatusSeverity.OK 
                                            ? "lime" : d.status.severity === StatusSeverity.WARNING 
                                            ? "yellow" : "red" 
                                        }}>
                                            {d.status.severity}
                                        </span> ] {d.status.message}
                                    </p>
                                </div>
                            )
                        })
                    }
                </ul>
            </Anime>
        </div>
    )
}
