"use client";

import React from "react";

class TextNode {
    private time_elapsed: number = 0;
    private last_time: number = Date.now();
    public duration: number;
    private finished_text: string;
    private loading_text: string;
    public active: boolean = false;
    private status: "OK" | "WARN" | "FAIL" = "OK";
    public decorational: boolean = false;
    
    constructor(
        duration: number, 
        loading_text: string, 
        finished_text: string | undefined = undefined, 
        decorational: boolean =false
    ) {
        this.duration = duration;
        this.loading_text = loading_text;
        this.finished_text = finished_text ?? loading_text;
        this.decorational = decorational;
    }

    public finish_now() {
        this.time_elapsed = this.duration;
    }

    public get_text() {
        return this.is_finished() ? this.finished_text : this.loading_text
    }

    public tick() {
        if (this.is_finished()) {
            return;
        }
        const now = Date.now();

        const delta = now - this.last_time;

        this.time_elapsed += delta;
        this.last_time = now;
    }

    public is_finished(): boolean {
        return this.time_elapsed >= this.duration;
    }

    public start() {
        this.active = true;

        this.last_time = Date.now();
    }

    public get_status_decorator() {
        if (this.decorational) {
            return "";
        }

        // [      ] (empty)
        // (looping) [***  ] [****  ] [ **** ] [  ****] [   ***] (and 0.3 delay before looping, 0.2 delay interframe)
        // [  OK  ]
        // [ WARN ]
        // [ FAIL ]
        // Use colors for the text, but not brackets - brackets stay white.
        // Loading dots are orange, and the leftmost and rightmost are yellow

        switch (this.is_finished()) {
            case true:
                switch (this.status) {
                    case "OK":
                        return (
                            <span>[ &nbsp;<span className="text-green-500">OK</span> &nbsp;]</span>
                        );
                    case "WARN":
                        return (
                            <span>[ <span className="text-yellow-500">WARN</span> ]</span>
                        );
                    case "FAIL":
                        return (
                            <span>[ <span className="text-red-500">FAIL</span> ]</span>
                        );
                }
            case false:
                // Run the [***   ] [****  ] [ **** ] [  ****] [   ***] animation
                const seconds = this.time_elapsed / 100;
                // Divisible into 5 phases each second
                let phase = Math.floor(seconds % 7) - 1;
                if (phase === -1) {
                    phase = 0;
                }
                if (phase === 5) {
                    phase = 4;
                }

                const half = Math.floor(seconds % 14);
                // Flip the phase if it's in the second half
                if (half >= 7) {
                    phase = 4 - phase;
                }

                let spaces_left = 0;
                let spaces_right = 0;
                let dots = 0;

                switch (phase) {
                    case 0:
                        spaces_left = 0;
                        spaces_right = 3;
                        dots = 3;
                        break;
                    case 1:
                        spaces_left = 0;
                        spaces_right = 2;
                        dots = 4;
                        break;
                    case 2:
                        spaces_left = 1;
                        spaces_right = 1;
                        dots = 4;
                        break;
                    case 3:
                        spaces_left = 2;
                        spaces_right = 0;
                        dots = 4;
                        break;
                    case 4:
                        spaces_left = 3;
                        spaces_right = 0;
                        dots = 3;
                        break;
                }

                return (
                    <span>
                        [<span>{"\xa0".repeat(spaces_left)}</span>
                        {<span className="text-orange-500">{"*".repeat(dots)}</span>}
                        <span>{"\xa0".repeat(spaces_right)}</span>]
                    </span>
                );
        }
    }
}

// Decoration "loading" messages (fill with random things to do that aren't actually loading, etc.)
// Style it like it's an Init System (E.g. SystemD, OpenRC, etc. It isn't, but it looks like it)
const textNodes: TextNode[] = [
    new TextNode(1000, `
\xa0___\xa0\xa0___\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0__\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0___\xa0
|__\xa0\xa0|__\xa0\xa0|\xa0\xa0\xa0\xa0|\xa0|\xa0\xa0\\\xa0\xa0/\\\xa0\xa0|__\xa0\xa0
|\xa0\xa0\xa0\xa0|___\xa0|___\xa0|\xa0|__/\xa0/~~\\\xa0|___\xa0
\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0
`, undefined, true),
    new TextNode(2000, "Starting services", "Started services"),
    new TextNode(1000, "Checking for updates", "No updates found"),
    // Begin the jokes here.
];

// These are comic ones.
// No jokes about destructive commands, etc.
const randomTextNodes: TextNode[] = [
    new TextNode(1000, "Loading the loading screen", "Loaded the loading screen"),
    new TextNode(500, "Checking for updates", "Found updates, but they're not important"),

    // Math
    new TextNode(500, "Counting past infinity", "Counted past infinity"),
    new TextNode(500, "Solving P vs NP", "Solved P vs NP"),
    new TextNode(500, "Finding the last digit of pi", "Found the last digit of pi"),
    new TextNode(500, "Dividing by zero", "Divided by zero"),

    // Science
    new TextNode(500, "Creating a black hole", "Created a black hole"),
    new TextNode(500, "Finding the Higgs Boson", "Found the Higgs Boson"),
    new TextNode(500, "Creating a new element", "Created a new element"),
    new TextNode(500, "Creating a new universe", "Created a new universe"),


    // Computer Science
    new TextNode(500, "Rewriting it in Rust", "Rewrote it in Rust"),
    new TextNode(1000, "Generating a random number between " + Math.floor(Math.random() * 100000) + " and " + Math.floor(Math.random() * 100000)),
    new TextNode(500, "Downloading more RAM", "Downloaded more RAM"),
    new TextNode(500, "Compiling the kernel", "Compiled the kernel"),
    new TextNode(500, "Running Arch Linux", "Ran Arch Linux (btw)"),
    new TextNode(500, "Compiling Gentoo", "Compiled Gentoo"),
    new TextNode(500, "Running Windows Update", "Ran Windows Update"),
    
    // - Programming
    new TextNode(500, "Writing a new programming language", "Wrote a new programming language"),
    

    // Mundane
    new TextNode(500, "Finding the meaning of life", "Found the meaning of life"),
    new TextNode(500, "Making a cup of tea", "Made a cup of tea"),
    new TextNode(500, "Taking a break", "Took a break"),
    new TextNode(500, "Doing something else", "Did something else"),
    new TextNode(500, "Changing the world", "Changed the world"),
    new TextNode(500, "Making a sandwich", "Made a sandwich"),
    new TextNode(500, "Doing absolutely nothing", "Did absolutely nothing"),

    // Occult
    new TextNode(500, "Summoning Cthulhu", "Summoned Cthulhu"),
    new TextNode(500, "Summoning Satan himself", "Summoned Satan himself"),
    new TextNode(500, "Summoning a demon", "Summoned a demon"),

    // LGBTQ+
    new TextNode(500, "Supporting LGBTQ+ rights", "Supported LGBTQ+ rights"),
    new TextNode(500, "Respecting pronouns", "Respected pronouns"),

    // Game references
    new TextNode(500, "Running the Konami Code", "Ran the Konami Code"),
    new TextNode(500, "Running the Doom cheat codes", "Ran the Doom cheat codes"),
    new TextNode(500, "Teleporting bread", "Teleported bread"),
    new TextNode(500, "Building a sentry", "Built a sentry"),
    new TextNode(500, "Building a dispenser", "Built a dispenser"),
    new TextNode(500, "Building a teleporter", "Built a teleporter"),
    new TextNode(500, "Spy checking", "Checked for spies"),
    new TextNode(500, "Rocket jumping", "Jumped with rockets"),
    new TextNode(500, "Demoknighting", "Demoknighted"),
    new TextNode(500, "Waiting for random crits", "Got random crits"),

    // Pop culture references
    new TextNode(500, "Finding the Holy Grail", "Found the Holy Grail"),
    new TextNode(500, "Finding the One Ring", "Found the One Ring"),
    new TextNode(500, "Finding the Death Star plans", "Found the Death Star plans"),
    new TextNode(500, "Finding the Infinity Stones", "Found the Infinity Stones"),
    new TextNode(500, "Finding the Ark of the Covenant", "Found the Ark of the Covenant"),
    new TextNode(500, "Finding the Philosopher's Stone", "Found the Philosopher's Stone"),

    // References towards franchises
    new TextNode(500, "Containing SCP-173", "Contained SCP-173"),

    // Joke programs that do weird, though not destructive or harmful things
    new TextNode(5000, "Running cowsay", "Ran cowsa- Moo."),
];

const postNodes = [
    new TextNode(10000, "Entering site now", "Entered site"),
];

// Randomly shuffle randomTextNodes into textNodes, but leave the elements already in textNodes in place
// Only pop in ~50% of the randomTextNodes so as to have some variety
const randomTextNodesLength = randomTextNodes.length / 2;
// Use to avoid duplicates
const pushedRandomTextNodes: Array<number> = [];
for (let i = 0; i < randomTextNodesLength; i++) {
    let index = Math.floor(Math.random() * randomTextNodes.length);
    while (pushedRandomTextNodes.includes(index)) {
        index = Math.floor(Math.random() * randomTextNodes.length);
    }
    pushedRandomTextNodes.push(index);
    const duration = randomTextNodes[index].duration * Math.random();
    randomTextNodes[index].duration = duration;
    textNodes.push(randomTextNodes[index]);
}

for (let i = 0; i < postNodes.length; i++) {
    textNodes.push(postNodes[i]);
}

const speedMultiplier = 0.1;

// Multiply the duration of each textNode by the speedMultiplier
for (let i = 0; i < textNodes.length; i++) {
    textNodes[i].duration *= speedMultiplier;
}

export default function IntroAnim(props: { onFinish: () => void }) {
    // Run the textNodes tick function every frame for each textNode
    const [textNodesState, setTextNodesState] = React.useState(textNodes);
    React.useEffect(() => {
        // Ensure only one interval is running (textNode.active)
        const interval = setInterval(() => {
            // Mark the last textNode active if
            // - it's not active
            // - it's not finished
            // - the previous textNode is finished
            for (let i = 0; i < textNodesState.length; i++) {
                const t = textNodesState[i];
                if (!t.active && !t.is_finished() && (i === 0 || textNodesState[i - 1].is_finished())) {
                    t.start();
                    break;
                }
            }

            // Tick all active textNodes
            for (let i = 0; i < textNodesState.length; i++) {
                const t = textNodesState[i];
                if (t.active) {
                    t.tick();
                }
            }

            if (textNodesState[textNodesState.length - 1].is_finished()) {
                props.onFinish();
            }

            // Update the state
            setTextNodesState([...textNodesState]);
        }, 16);

        return () => clearInterval(interval);
    });
    
    return (
        <div className="vga-font">
            <div className="text-left">
                {
                    textNodesState.filter(t => t.active || t.is_finished()).map((t) => (
                        <div style={
                            {
                                lineHeight: "1",
                                whiteSpace: 'pre-line'
                            }
                        } key={t.get_text()}>
                            {t.get_status_decorator()}
                            <span> </span>
                            {t.get_text()}
                        </div>
                    ))
                }
            </div>
        </div>
    );
}
